#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, string, re, copy
import os, fnmatch
import codecs

from qt import Qt, QHBoxLayout, QVBoxLayout, SIGNAL, SLOT, QLabel, QWidget, QToolTip, QDialog, QColor
from kdecore import KApplication
from kdeui import KPushButton, KTextEdit, KTabWidget
from kspell import KSyntaxHighlighter

class Highlighter(KSyntaxHighlighter):
    def set_pattern(self,patternlist, patternlist2=[]):
        self.patternlist,self.patternlist2 = patternlist,patternlist2
    def highlightParagraph(self, text, endStateOfLastPara):
        for pattern in self.patternlist:
            it=re.finditer(pattern, unicode(text))
            for i in it:
                self.setFormat (i.start(),len(i.group()), Qt.red) #QColor(255,0,0))
        for pattern in self.patternlist2:
            it=re.finditer(pattern, unicode(text))
            for i in it:
                self.setFormat (i.start(),len(i.group()), Qt.green) #QColor(0,255,0))
        return 0

class MainDialog(QDialog):
    def __init__(self, parent=None, pofilenrpatternlist=[], title=None):
        self.desktopfilestrings=["Name=", "Comment=", "GenericName="] #kde3
        self.liste=pofilenrpatternlist #[[pofile, nr, patternlist]. [pofile, nr, patternlist],...]
        self.nr, self.maxnr=0, len(self.liste)-1
        self.listmsgstrorig, self.listfuzzyorig=[],[]
        for i in range(len(self.liste)):
            self.listmsgstrorig.append(self.liste[i][0].msgstr[self.liste[i][1]])
            self.listfuzzyorig.append(self.liste[i][0].is_fuzzy(self.liste[i][1]))
        self.listmsgstrchanged=copy.copy(self.listmsgstrorig)
        self.listfuzzychanged=copy.copy(self.listfuzzyorig)
#        plural liste mit 2 msgstr !!!!!!
        QDialog.__init__(self, parent)
        self.setCaption(title)

        self.nolabel= QLabel('no x of y messages',self)
        self.namenrlabel= QLabel('po-file #no',self)
        self.pattern_label_widget()
        self.ctxtlabel= QLabel('context: ctxt',self)
        self.msgstrlabel= QLabel('msgstr: fuzzy',self)
        self.prevbutton = KPushButton('<',self)
        self.prevbutton.setAccel(Qt.Key_Prior)
        tooltip = QToolTip.add(self.prevbutton,'Goto previous item (Page Up)')
        self.nextbutton = KPushButton('>', self)
        self.nextbutton.setAccel(Qt.Key_Next)
        tooltip = QToolTip.add(self.nextbutton,'Goto next item (Page Down)')
        self.firstbutton = KPushButton('<<', self)
        self.firstbutton.setAccel(Qt.Key_Home)
        tooltip = QToolTip.add(self.firstbutton,'Goto first item (Home)')
        self.lastbutton = KPushButton('>>', self)
        self.lastbutton.setAccel(Qt.Key_End)
        tooltip = QToolTip.add(self.lastbutton,'Goto last item (End)')
        self.closebutton = KPushButton('&Save && Quit', self)
        tooltip = QToolTip.add(self.closebutton,'Close and save all changed messages')
        self.quitbutton = KPushButton('&Discard && Close', self)
        tooltip = QToolTip.add(self.quitbutton,'Quit and discard all changed messages')
        self.revertbutton = KPushButton('&Revert all', self)
        tooltip = QToolTip.add(self.revertbutton,'revert all changes to recent message')
        self.fuzzybutton = KPushButton('Fuzzy', self)
        #self.fuzzybutton.setAccel(Qt.CTRL+Qt.Key_U) geht nicht - automatisch &Fuzzy gesetzt
        tooltip = QToolTip.add(self.fuzzybutton,'Toggle fuzzy status of recent message')#(Ctrl+U)')

        self.msgid = KTextEdit('msgid', '', self)
        self.msgid.setTextFormat(Qt.PlainText)
        self.msgid.setReadOnly (True)
        self.msgid1 = KTextEdit('msgid plural', '', self)
        self.msgid1.setTextFormat(Qt.PlainText)
        self.msgid1.setReadOnly (True)
        self.msgidtab=KTabWidget(self)
        self.msgidtab.insertTab (self.msgid,'Plural 1')
        self.msgidtab.insertTab (self.msgid1,'Plural 2')

        self.msgstr = KTextEdit('msgstr\nsecondline', '', self)
        self.msgstr.setTextFormat(Qt.PlainText)
        self.msgstr1 = KTextEdit('msgstr plural\nsecondline', '', self)
        self.msgstr1.setTextFormat(Qt.PlainText)
        self.msgstrtab=KTabWidget(self)
        self.msgstrtab.insertTab (self.msgstr,'Plural 1')
        self.msgstrtab.insertTab (self.msgstr1,'Plural 2')
        self.set_highlighters()
        layout = QVBoxLayout(self, 8)
        layout.addWidget(self.nolabel)
        layout.addWidget(self.namenrlabel)
        layout.addWidget(self.patternwidget)
        layout.addWidget(self.ctxtlabel)
        layout.addWidget(QLabel('msgid:',self))
        layout.addWidget(self.msgidtab)
        layout.addWidget(self.msgstrlabel)
        layout.addWidget(self.msgstrtab)
        buttonlayout = QHBoxLayout(layout, 4)
        buttonlayout.addWidget(self.prevbutton)
        buttonlayout.addWidget(self.nextbutton)
        buttonlayout.addWidget(self.fuzzybutton)
        buttonlayout.addWidget(self.closebutton)
        button2layout = QHBoxLayout(layout, 4)
        button2layout.addWidget(self.firstbutton)
        button2layout.addWidget(self.lastbutton)
        button2layout.addWidget(self.revertbutton)
        button2layout.addWidget(self.quitbutton)
#        layout.addStretch(1)
        self.connect(self.firstbutton, SIGNAL('clicked()'), self.firstItem)
        self.connect(self.prevbutton, SIGNAL('clicked()'), self.previousItem)
        self.connect(self.nextbutton, SIGNAL('clicked()'), self.nextItem)
        self.connect(self.lastbutton, SIGNAL('clicked()'), self.lastItem)
        self.connect(self.closebutton, SIGNAL('clicked()'), self,SLOT('accept()'))
        self.connect(self.quitbutton, SIGNAL('clicked()'), self,SLOT('reject()'))
        self.connect(self.revertbutton, SIGNAL('clicked()'), self.revertchanges)
        self.connect(self.fuzzybutton, SIGNAL('clicked()'), self.togglefuzzy)
        self.connect(self.msgstr, SIGNAL('textChanged()'), self.textChanged)
        self.connect(self.msgstr1, SIGNAL('textChanged()'), self.textChanged)
        self.connect(self.msgidtab, SIGNAL('currentChanged(QWidget *)'), self.currentmsgidPageChanged)
        self.connect(self.msgstrtab, SIGNAL('currentChanged(QWidget *)'), self.currentmsgstrPageChanged)
        self.setdata()
    def pattern_label_widget(self):
        self.patternwidget= QLabel('pattern:\n*',self)
        self.patternwidget.setAlignment(Qt.WordBreak)
        self.patternwidget.setTextFormat(Qt.PlainText)
    def currentmsgidPageChanged(self,newpage):
        if self.msgstrtab.currentPageIndex()!=self.msgidtab.currentPageIndex(): #wg rekursion
            self.msgstrtab.setCurrentPage(self.msgidtab.currentPageIndex())
    def currentmsgstrPageChanged(self,newpage):
        if self.msgidtab.currentPageIndex()!=self.msgstrtab.currentPageIndex():
            self.msgidtab.setCurrentPage ( self.msgstrtab.currentPageIndex() )
    def set_highlighters(self):
        self.highlighter = Highlighter(self.msgstr)
        self.highlighter1 = Highlighter(self.msgstr1)
        self.listhighlighters=[self.highlighter,self.highlighter1]
    def firstItem(self):
        self.set_msgstrpo()
        self.nr=0
        self.setdata()
    def previousItem(self):
        if self.nr>0:
            self.set_msgstrpo()
            self.nr-=1
            self.setdata()
    def nextItem(self):
        if self.nr<self.maxnr:
            self.set_msgstrpo()
            self.nr+=1
            self.setdata()
    def lastItem(self):
        self.set_msgstrpo()
        self.nr=self.maxnr
        self.setdata()
    def accept(self):
        self.set_msgstrpo() #für Änderung in aktueller message
        #hier erst alle changes in po files eintragen
        for i in range(len(self.liste)):
            if self.listmsgstrchanged[i]!=self.listmsgstrorig[i]:
                self.liste[i][0].set_msgstr(self.liste[i][1],self.listmsgstrchanged[i])
            # fuzzy nach msgstr, damit msgstr + fuzzy modify geht
            if self.listfuzzychanged[i]!=self.listfuzzyorig[i]:
                if self.listfuzzychanged[i]:
                    self.liste[i][0].set_fuzzy(self.liste[i][1])
                else:
                    self.liste[i][0].remove_fuzzy(self.liste[i][1])
        QDialog.accept(self)
    def revertchanges(self):
        print "message %d: revert all changes of msgstr and fuzzy status" %(self.nr+1)
        self.listfuzzychanged[self.nr]=self.listfuzzyorig[self.nr]
        self.listmsgstrchanged[self.nr]=self.listmsgstrorig[self.nr]
        self.setdata()
    def togglefuzzy(self):
        self.listfuzzychanged[self.nr]=not self.listfuzzychanged[self.nr]
        print "set to fuzzy = %s" %self.listfuzzychanged[self.nr]
        self.setmsgstrlabeltext()
    def textChanged(self):
        if self.msgstr.isModified() or self.msgstr1.isModified():#listmsgstrorig!=listmsgstrchanged ???
            if self.listfuzzychanged[self.nr]==True:
                self.togglefuzzy()
            self.setmsgstrlabeltext()
    def setdata(self):
        self.nolabel.setText("no %d of %d messages:" %(self.nr+1,self.maxnr+1))
        k=self.liste[self.nr]
        self.namenrlabel.setText('%s/%s/%s #%d' %(k[0].filetype,k[0].modulname,k[0].name, k[1]))
        self.set_patterntext(k)
        ctxt=k[0].msgctxt[k[1]]
        self.ctxtlabel.setText("context: %s"%ctxt)
        if k[0].is_plural(k[1]):
            self.msgidtab.setTabBarHidden(False)
            self.msgstrtab.setTabBarHidden(False)
            msgid,msgid1=k[0].msgid[k[1]][0],k[0].msgid[k[1]][1]
            msgstr,msgstr1=self.listmsgstrchanged[self.nr][0],self.listmsgstrchanged[self.nr][1]
            self.msgid.setText(msgid)
            self.msgid1.setText(msgid1)
            self.msgstr.setText(msgstr)
            self.msgstr1.setText(msgstr1)
        else:
            #msgid und msgstr aktivieren koppeln!!!
            self.msgidtab.setCurrentPage(0)
            self.msgstrtab.setCurrentPage(0)
            self.msgidtab.setTabBarHidden(True)
            self.msgstrtab.setTabBarHidden(True)
            msgid,msgstr=k[0].msgid[k[1]],self.listmsgstrchanged[self.nr]
            self.msgid.setText(msgid)
            self.msgstr.setText(msgstr)
        if self.nr<self.maxnr:
            self.nextbutton.setEnabled(True)
            self.lastbutton.setEnabled(True)
        else:
            self.nextbutton.setEnabled(False)
            self.lastbutton.setEnabled(False)
        if self.nr>0:
            self.prevbutton.setEnabled(True)
            self.firstbutton.setEnabled(True)
        else:
            self.prevbutton.setEnabled(False)
            self.firstbutton.setEnabled(False)
        self.setmsgstrlabeltext()
    def setmsgstrlabeltext(self):
        self.revertbutton.setEnabled(False)
        txt="msgstr: - fuzzy=%s" %self.listfuzzychanged[self.nr]
        if self.listfuzzychanged[self.nr]!=self.listfuzzyorig[self.nr]:
            txt+=" modified"
            self.revertbutton.setEnabled(True)
        if self.get_msgstrtxt()!=self.listmsgstrorig[self.nr]:
            txt+=" - msgstr modified"
            self.revertbutton.setEnabled(True)
        self.msgstrlabel.setText(txt)
    def set_patterntext(self,k):
        patterntext,pattern,patternnr='pattern: ',[],1
        for i in k[2]:
            if patternnr>1:patterntext+=" "*13
            patterntext+='>%s< %s\n' %(i[0],i[1])
            pattern.append(i[0])
            patternnr+=1
        for i in self.listhighlighters: i.set_pattern(pattern)
        self.patternwidget.setText(patterntext)
    def get_msgstrtxt(self):#plural ok
        if self.liste[self.nr][0].is_plural(self.liste[self.nr][1]):
            m=[unicode(self.msgstr.text()),unicode(self.msgstr1.text())]
        else: m=unicode(self.msgstr.text())
        if self.liste[self.nr][0].is_desktop_file(): # Name=, Comment= etc in desktop files 
            for i in self.desktopfilestrings:
                if m[:len(i)]==i: # == Name=, Comment= usw
                    m=m[len(i):]  # weg damit
        return m
    def set_msgstrpo(self): # plural ok
        if self.msgstr.isModified() or self.msgstr1.isModified():#ist unnötig? geht nicht mit Undo aus contxt menu?
            m=self.get_msgstrtxt()
            if self.liste[self.nr][0].is_desktop_file():#hier gibts kein plural
                for j in self.desktopfilestrings:  #desktop_ files Name= Comment= GenericName=
                     if self.listmsgstrchanged[self.nr][0:len(j)]==j:
                        m=j+m
            if self.listmsgstrorig[self.nr]!=m:
                print "will change >%s< to >%s<" %(self.listmsgstrorig[self.nr], m)
                self.listmsgstrchanged[self.nr]=m
            if self.listfuzzychanged[self.nr]!=self.listfuzzyorig[self.nr]:
                print "will change fuzzy status to %s" %self.listfuzzychanged[self.nr]

class WordDialog(MainDialog):
    def set_patterntext(self,k):
        patterntext=''
        en,default,wrong=[],[],[]
        for i in range(len(k[2][0])):
            en.append(k[2][0][i])
            default+=k[2][1][i]
            wrong+=k[2][2][i]
            patterntext+='word: %s default: %s' %(k[2][0][i],','.join(k[2][1][i]))
            if wrong!= ['']:
                patterntext+=' wrong: %s\n' %','.join(k[2][2][i])
            else:
                patterntext+='\n'
        self.highlighterid.set_pattern(en)
        self.highlighterid1.set_pattern(en)
        self.highlighterstr.set_pattern(wrong,default)
        self.highlighterstr1.set_pattern(wrong,default)
        self.patternwidget.setText(patterntext)
    def set_highlighters(self):
        self.highlighterid = Highlighter(self.msgid)
        self.highlighterid1 = Highlighter(self.msgid1)
        self.highlighterstr = Highlighter(self.msgstr)
        self.highlighterstr1 = Highlighter(self.msgstr1)

def _filter(s,pattern):
    such=re.compile(pattern,re.DOTALL)
    smarkup=such.findall(s)
    for i in smarkup:
        s=s.replace(i,'')
    return s
def markupfilter(s):
#	for i in ['menuchoice','guisubmenu','guimenu','guimenuitem','guilabel','guibutton','guiicon',\
#	          'keycap','keysym','keyword','shortcut','keycombo','keycombo action="simul"']:
#            s=s,replace('<%s>'%i,'')
#            s=s.replace('</%s>'%i,'')
    return _filter(s,'<.*?>')
def entityfilter(s):
    return _filter(s,'&.*?;')

def getfilelistfromdir(arg1, pattern):
    filelist=[]
    if os.path.isfile(arg1) and pattern in arg1:
        filelist.append(arg1)
    else:
        if os.path.isdir(arg1) and arg1[-1]!="/":
            arg1+="/"
        muster=os.path.basename(arg1)+"*"+pattern
        for root,dirs,files in os.walk(os.path.dirname(arg1)):
            for name in files: 
                if not ".svn" in name:
                    if fnmatch.fnmatch(name,muster):
                        filelist.append(os.path.join(root, name))
    filelist.sort()
    return filelist

class l10nfile:
    def __init__(self,filename):
        filename=os.path.abspath(filename)
        if 'l10n-kde3/' in filename: l10n,self.kdelibsver='l10n-kde3/',''
        elif 'l10n-kde4/' in filename: l10n,self.kdelibsver='l10n-kde4/','4'
        else: l10n,self.kdelibsver='l10n/',''
        self.filename=filename
        self.name=os.path.basename(filename)
        #/path/to/l10n/lang/messages/modul/foo.po
        #/path/to/l10n/lang/docmessages/modul/foo[_bar].po
        #/path/to/l10n/lang/docmessages/modul/foo[_man-bar].po
        #/path/to/l10n/documentation/modul/foo/bar.docbook
        try:
            self.svndir,rest=filename.split(l10n)
            self.l10nname=self.svndir+l10n
            if rest.split('/')[0] == 'documentation':
                try:
                    self.filetype,self.modulname,self.appname,rest=rest.split('/')
                except: #kdeaddons
                    self.filetype,self.modulname,self.submodulname,self.appname,rest=rest.split('/')
                self.langname,self.pot='templates','t'
            else:
                try:
                    self.langname,self.filetype,self.modulname,rest=rest.split('/')
                except: #kdeaddons
                    self.langname,self.filetype,self.modulname,self.submodulname,rest=rest.split('/')
                appname=rest.split('.po')[0]
                self.appname=appname.split('_')[0]
                self.pot=''
        except:
            self.svndir,self.l10nname,self.filetype,self.modulname,self.appname,self.pot='','','','','',''
        self.load()

class docbookdocmessage(l10nfile):
    def find_message_catalogues(self):
        messagedir=self.l10nname+self.langname+'/messages/'
        self.generalcataloglist=[messagedir+'qt/kdeqt.po'+self.pot]#falsch für l10n-kde3
        self.generalcataloglist.append(messagedir+'kdelibs/kdelibs'+self.kdelibsver+'.po'+self.pot)
        self.generalcataloglist.append(messagedir+'kdelibs/kio'+self.kdelibsver+'.po'+self.pot)
        self.generalcataloglist.append(messagedir+'kdelibs/libphonon.po'+self.pot)
        if os.path.exists(messagedir+self.modulname+'/desktop_'+self.modulname+'.po'+self.pot):
            self.generalcataloglist.append(messagedir+self.modulname+'/desktop_'+self.modulname+'.po'+self.pot) 
            # wg extragear.multimedia 2 x desktop_ file -> gibt sogar module mit > 2 desktop_ files
            #Name= etc raus !!!!
        if self.modulname=='koffice':
            self.generalcataloglist.append(messagedir+self.modulname+'/koffice.po'+self.pot)
            if self.appname in ['kword','kspread','karbon']:
                self.generalcataloglist.append(messagedir+self.modulname+'/kofficefilters.po'+self.pot)
                #kfile_koffice.po, kfile_ooo.po, kscreenshot_plugin.po,kscan_plugin.po,kformdesigner.po,
                #koconverter.po, kounavail.po ???
        #alle libmodul.po	
        guidir=messagedir+self.modulname+'/'
        self.appcataloglist=getfilelistfromdir(guidir,'*lib'+self.modulname+'*.po'+self.pot)
        if self.appname in ['kttsd']:
            self.appcataloglist.append(guidir+'libKTTSD.po'+self.pot)
        if self.modulname in 'kdeaddons':
            if 'konq-plugins' in self.filename:
                print guidir,'*'+self.appname+'*.po'+self.pot
                self.appcataloglist+=getfilelistfromdir(guidir,'*.po'+self.pot) #nicht nötig, doch für de/docmess
                #+konqueror?
                self.appcataloglist.append(messagedir+'kdebase/konqueror.po'+self.pot)
            if 'kate-plugins' in self.filename:
                self.appcataloglist+=getfilelistfromdir(guidir,'*kate*.po'+self.pot)
                #+kate?
            if 'kicker-applets' in self.filename:
                self.appcataloglist+=getfilelistfromdir(guidir,'*kolourpicker*.po'+self.pot)
                self.appcataloglist+=getfilelistfromdir(guidir,'*ktimemon*.po'+self.pot)
                self.appcataloglist.append(messagedir+'kdebase/kicker.po'+self.pot) #+kicker
        if self.appname=='lilo-config':  #kdeadmin
            self.appcataloglist+=getfilelistfromdir(guidir,'*lilo*.po'+self.pot)
        if self.modulname in 'kdebase':
            # userguide alle *.po ?
            # if 'kcontrol' in arg1: geht über alle self.appname.po
            #   self.appcataloglist+=getfilelistfromdir(guidir+'/','*'+self.appname+'*.'+self.pot)
            # kdebase/kcontrol alle kcm*.po aus kdebase?
            if 'kcontrol' in self.filename:
                self.appcataloglist+=getfilelistfromdir(guidir,'kcm*.po'+self.pot)
            if 'infocenter' in self.filename:
                self.appcataloglist.append(guidir+'kcminfo.po'+self.pot) #?
            if 'kdeprint' in self.filename:
                self.appcataloglist.append(guidir+'kprinter.po'+self.pot)
                self.appcataloglist.append(guidir+'kjobviewer.po'+self.pot)
                self.appcataloglist.append(guidir+'kio_print.po'+self.pot)
                self.appcataloglist.append(guidir+'kcmprintmgr.'+self.pot)
                self.appcataloglist.append(messagedir+'kdelibs/ppdtranslations.po'+self.pot)
            if 'konqueror' in self.filename:
                self.appcataloglist.append(guidir+'libkonq.po'+self.pot) #?
            if 'kate' in self.filename:
                self.appcataloglist.append(messagedir+'kdelibs/katepart.po'+self.pot)
            if 'kwrite' in self.filename:
                self.appcataloglist.append(messagedir+'kdelibs/katepart.po'+self.pot)
                self.appcataloglist.append(guidir+'kate.po'+self.pot)
            if 'kicker' in self.filename:
                self.appcataloglist.append(guidir+'libtaskmanager.po'+self.pot)
                self.appcataloglist.append(guidir+'kasbarextension.po'+self.pot)
                self.appcataloglist.append(guidir+'dockbarextension.po'+self.pot)
                #self.appcataloglist.append(guidir+'/kickersidebarextension.'+self.pot)
                self.appcataloglist.append(guidir+'konqueror.po'+self.pot)
                self.appcataloglist.append(guidir+'libkonq.po'+self.pot)
                self.appcataloglist.append(guidir+'libdmctl.po'+self.pot)
                self.appcataloglist.append(guidir+'libtaskbar.po'+self.pot)
                self.appcataloglist.append(guidir+'mediaapplet.po'+self.pot)
                self.appcataloglist.append(guidir+'naughtyapplet.po'+self.pot)
                self.appcataloglist.append(guidir+'lockout.po'+self.pot)
                self.appcataloglist.append(guidir+'quicklauncher.po'+self.pot)
                self.appcataloglist.append(guidir+'trashapplet.po'+self.pot)
                self.appcataloglist.append(guidir+'kmenuapplet.po'+self.pot)
                self.appcataloglist.append(guidir+'krunapplet.po'+self.pot)
                self.appcataloglist.append(guidir+'ksystemtrayapplet.po'+self.pot)
                #self.appcataloglist.append(guidir+'/ktaskbarapplet.'+self.pot)
                self.appcataloglist.append(guidir+'clockapplet.po'+self.pot)
                self.appcataloglist.append(messagedir+'kdelibs/timezones.po'+self.pot)
                self.appcataloglist.append(guidir+'kminipagerapplet.po'+self.pot)
                #libtaskmanager, kasbarextension, dockbarextension, kickersidebarextension, konqueror, 
                #libkonq, libdmctl, libtaskbar,mediaapplet, naughtyapplet,  lockout,quicklauncher
                # trashapplet, kmenuapplet, krunapplet, ksystemtrayapplet, ktaskbarapplet,clockapplet,timezones,kminipagerapplet
        #alle self.appname.po
        #print guidir,'*'+self.appname+'*.'+self.pot,getfilelistfromdir(guidir,'*'+self.appname+'*.'+self.pot)
        self.appcataloglist+=getfilelistfromdir(guidir,'*'+self.appname+'*.po'+self.pot)

    def message_filter(self,s):
        #s=s.rstrip(' ...')         #' ...' am Ende ausfiltern fuer de
        #s=s.rstrip('...')          #'...' am Ende ausfiltern,
        s=s.rstrip(':')           #':' am ende raus ?
        #&amp; &lt; &gt; ersetzen durch & < >, noch andere? z.B. &quote; \" \' !!!!!!!!!
        s=string.replace(s,'&amp;','&')
        s=string.replace(s,'&lt;','<')
        s=string.replace(s,'&gt;','>')
        s=string.replace(s,'\n','')
        #s=string.replace(s,'\n',' ') # ok? nein
        #leerzeichen an anfang/ende + mehrfache in mitte hier ausfiltern?
        #satzzeichen am ende ausfiltern?
        return s
    def extract_guiitems(self,s,itemtyp=''):
        such=re.compile('<gui'+itemtyp+'.*?/gui'+itemtyp+'.*?',re.DOTALL) # mit \n drin #nur text zwischen '<gui*>'+'</gui*>'
        #such=re.compile('<gui.*?/gui.*?',re.DOTALL) # mit \n drin
        return such.findall(s)
    def get_itemtyp_sraw(self,s):
        itemtyp=string.splitfields(s,'>')[0][1:]
        itemtyp=itemtyp.replace('\n','') #wg nl vor > in docbooks
        s_unchanged=string.splitfields(string.splitfields(s,'>')[1],'</')[0] #falsch -> userguide
        return itemtyp,s_unchanged

class message_catalog(l10nfile):
    def __init__(self,filename,gettexttyp,backup=False):
        #self.fuzzystr='fuzzy' 
        self.filechanged=False
        self.gettexttyp=gettexttyp
        self.backup=backup
        l10nfile.__init__(self,filename)
    def load(self):
        f=codecs.open(self.filename,"rb","utf8").read()
        #s=f.split('\n\n') #geht nur mit leerzeile! gibts auch zeilen mit whitespace zwischen den nachrichten??
        f=f.rstrip()+'\n' # >1 whitespaces (newlines etc) am dateiende weg
        self.obsoletemsg=''
        try: f,self.obsoletemsg=f.split('\n#~ ',1) #falsch für "\n\n#, fuzzy\n#~ msgid ..."  "\n\n#, Tag foo\n#~ msgid ..."
        except: pass
        if self.obsoletemsg!='':self.obsoletemsg='#~ '+self.obsoletemsg
        s=re.split('\n[:space:]*\n+',f) #2 oder mehrere '\n\n' + `\n \n'
        if s[-1]!='' and not('msgstr' in s[-1] and 'msgid' in s[-1]): #s[-1] = #, fuzzy oder # Tag foo für obsolete
            self.obsoletemsg='\n'+s[-1]+'\n'+self.obsoletemsg
            s=s[:-1]
        self.msgstrraw, self.msgidraw, self.comraw =[], [], []
        self.msgstr, self.msgstrfilter, self.msgid, self.msgidfilter, = [], [], [], []
        self.msgidprev, self.msgidprevfilter, self.msgctxtprev, self.msgctxt, self.msgstrchanged=[], [], [], [], []
        for i in s:
            commsgid=i.split('\nmsgstr')[0]
            msgraw =i.split('\nmsgstr')[1:]
            com = commsgid.split('\nmsgid')[0]
            self.comraw.append(com)
            midraw = (commsgid.split('\nmsgid')[1:])
            if self.gettexttyp=='kde4':
                try:    com,ctxt=com.split("\nmsgctxt ")
                except: ctxt=''
                try:    com,prev=com.split("#| msgid ")
                except: prev=''
                try:    ctxtprev=com.split("#| msgctxt ")[1]
                except: ctxtprev=''
            else: prev, ctxtprev = '', ''
            #zeilen mit "\n" !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!????????????
            self.msgstrraw.append(msgraw)
            self.msgidraw.append(midraw)
            mid=''.join(midraw).lstrip()
            mid=mid.replace('\\"', "\\'")
            mid=mid.replace('\n','').replace('"','')
            mid=mid.replace("\\'", '\\"')
            if self.gettexttyp=='kde3': 
                ctxt,mid=self.filter_comment_from_msgid(mid)
            self.msgctxt.append(ctxt)
            self.msgctxtprev.append(ctxtprev)
#            self.msgidprev.append(prev) #raw?
            prev=prev.replace('\\"', "\\'").replace('\n#| ','').replace('"','').replace("\\'", '\\"')
            self.msgidprev.append(prev)
            self.msgidprevfilter.append(self.message_filter(prev))#funktioniert nicht mit plural
            self.msgidfilter.append(self.message_filter(mid)) #funktioniert nicht mit plural
            msg=''.join(msgraw).lstrip()
            msg=msg.replace('\\"', "\\'")
            msg=msg.replace('\n','').replace('"','')
            msg=msg.replace("\\'", '\\"')
            self.msgstrfilter.append(self.message_filter(msg))#funktioniert nicht mit plural
            if self.is_plural(len(self.msgidraw)-1): 
                mid,msg=self.split_plural(mid,msg)
            self.msgid.append(mid)
            self.msgstr.append(msg)
            self.msgstrchanged.append(False)
    def split_plural(self,mid,msg):
        if self.gettexttyp=='kde3': 
            mid,msg=mid.lstrip('_n: ').split('\\n'), msg.split('\\n')
        else:
            mid,msg=mid.split('_plural '), re.split('\[[0-9]\] ',msg)[1:]
        return mid,msg
    def filter_comment_from_msgid(self,msgid): #kde3
        if msgid.startswith('_: '):
            try:
                comid,msgid=msgid[3:].split('\\n',1) #split at first \n
            except:
                comid,msgid=msgid[3:],'' #e.g. kdelibs.po '"_: Dear Translator! Translate this string ...' without \n
        else:
            comid=''
        return comid,msgid
    def messagerange(self):
        return range(1,len(self.comraw))
    def set_fuzzy(self,nr):
        if not self.is_fuzzy(nr): 
            if '#, ' in self.comraw[nr]:
                self.comraw[nr]=self.comraw[nr].replace('#, ','#, fuzzy, ') #, fuzzy, foo
            elif '#| msgctxt' in self.comraw[nr]:self.comraw[nr]=self.comraw[nr].replace('#| msgctxt','#, fuzzy\n#| msgctxt') #, fuzzy
            elif '#| msgid ' in self.comraw[nr]:self.comraw[nr]=self.comraw[nr].replace('#| msgid ','#, fuzzy\n#| msgid ') #, fuzzy
            else: self.comraw[nr]+='\n#, fuzzy' #, fuzzy
            self.filechanged=True
    def remove_fuzzy(self,nr):
        if self.is_fuzzy(nr): 
            self.comraw[nr]=self.comraw[nr].replace('fuzzy, ','') #, fuzzy, foo
            self.comraw[nr]=self.comraw[nr].replace('\n#, fuzzy','')#, fuzzy
            self.filechanged=True
    def is_fuzzy(self, nr):
        if 'fuzzy' in self.comraw[nr]:
           return True
        else:
           return False
    def is_plural(self, nr):
      try: # wg kde3 messages/kdelibs/kmcop.po
        if len(self.msgidraw[nr])>1 or self.msgidraw[nr][0].startswith(' ""\n"_n: '): #kde4 or kde3
           return True
        else:
           return False
      except:return False
    def concat_pofile(self):
        l=[]
        for i in range(len(self.comraw)):
            l.append(self.comraw[i])
            for j in self.msgidraw[i]:  l.append('\nmsgid'+j)
            if self.msgstrchanged[i]:
                l.append('\n'+self.msgstrraw[i]+'\n')
                self.msgstrchanged[i]=False
            else:
                for j in self.msgstrraw[i]: l.append('\nmsgstr'+j)
                l.append('\n\n')
        l[-1]=l[-1].rstrip('\n')
        if self.obsoletemsg!='':l.append('\n'+self.obsoletemsg)
        return ''.join(l)
    def saveclose(self):
        if self.filechanged:
            if self.backup:
                os.rename(self.filename,self.filename+'.backup')
            f=codecs.open(self.filename,'w',"utf8")
            f.write(self.concat_pofile())
            f.close()
            self.filechanged=False
#	    return True
#	else return False
    def format_msgstr(self,nr,m): #siehe auch msgsplit, das arbeitet anders!!! msgstr + msgstrfilter aendern!!!!!!!!!!!!!!
            if len(m)<=78:
                self.msgstrraw[nr]='msgstr "%s"\n' %m
            else:
                self.msgstrraw[nr]='msgstr ""\n'
                mwords=string.split(m,' ')
                a=0
                rest=''
                for j in range(len(mwords)):
                    if len(' '.join(mwords[a:j+1]))>78: 
                        self.msgstrraw[nr]+='"%s "\n' %(' '.join(mwords[a:j+1]))
                        a=j+1
                        rest=' '.join(mwords[j+1:])
                if rest!='':
                    self.msgstrraw[nr]+='"%s"\n' %rest
    def set_msgstr(self,nr,s):
        try:
            s=unicode(s,'utf-8')
            svorh=unicode(self.msgstr[nr],'utf-8')
        except:
           svorh=self.msgstr[nr]
        if s!=svorh:
            if self.is_plural(nr):
                if self.gettexttyp=='kde3':
                    self.msgstrraw[nr]='msgstr ""\n"%s\\n"\n"%s"\n' %(s[0],s[1])
                else:
                    self.msgstrraw[nr]='msgstr[0] "%s"\nmsgstr[1] "%s"\n' %(s[0],s[1])
            else:
                self.format_msgstr(nr,s)
            self.remove_fuzzy(nr)
            self.filechanged=True
            self.msgstrchanged[nr]=True
    def is_translation(self,nr): #kde3 + kde4
        if self.msgid[nr] in ['Your emails','Your names','ROLES_OF_TRANSLATORS','CREDIT_FOR_TRANSLATORS']:
            return False
        else:
            return True
    def is_translated(self,nr):
        if string.join(self.msgstr[nr],'') not in ['']: #wg plural
            return True
        else:
            return False

class docpo(message_catalog,docbookdocmessage):
    def __init__(self,filename,gettexttyp,backup=False):
        message_catalog.__init__(self,filename,gettexttyp,backup)
        docbookdocmessage.__init__(self,filename)
#        self.find_message_catalogues()
#    def guiitems_in_msgid_msgstr(self,nr):
#        for i in ['guibutton','guimenu','guisubmenu','guimenuitem','guilabel','guiicon']:
#            if itemtyp in self.commid(nr): #gui? in com
#                self.guiitems_msgid='<'+i+'>'+self.msgid(nr)+'</'+i+'>'
#                self.guiitems_msgstr=self.guiitems_msgid
#        self.guiitems_msgid+=self.extract_guiitems(self.msgid(nr),itemtyp)
#        self.guiitems_msgstr+=self.extract_guiitems(self.msgstr(nr),itemtyp)
    def message_filter(self,s):
        smarkup=re.findall('> *?<',s) #'<foo> <foo>' -> '<foo><foo>
        for i in smarkup:
            s=s.replace(i,'><')
        return s
    def is_desktop_file(self):
        return False

class guipo(message_catalog):
    #def __init__(self,filename,backup=False):
        #message_catalog.__init__(self,filename,backup)
    def message_filter(self,s):
        if self.is_desktop_file(): #nur für kde3 nötig
           try:
               s=s.split('=')[1] #'Name=' etc raus FIXME besser .split('Name=') etc
           except: pass
        s=s.replace('&amp;','xxxx')      # &amp->xxxx wegen clash &a, mit && '&Header && Footer...'->'Header Footer...'
        s=s.replace('&&','xxxx')
        s=s.replace('&lt;','<')        # &lt->< wegen clash &l
        s=s.replace('&gt;','>')        # &gt->> wegen clash &g
        for i in re.findall('&[a-z,A-Z,0-9,Ü,ü,Ö,ö,ä,Ä]',s):   # alle '&[a-z,A-Z,0-9]' ausfiltern, Umlaute in utf8?
            #print s,'clash'
            s=s.replace(i,i[1])        #i='&a',i[1]='a'  # = Clash
        s=s.replace('xxxx','&')                # alle xxxx -> &
        #s=s.rstrip(' ...')           #' ...' am Ende ausfiltern fuer de
        #s=s.rstrip('...')            #'...' am Ende ausfiltern
        s=s.rstrip(':')           #':' am ende raus ?
	#s=s.replace('\n','')
        if '</' in s:
            for i in ['<qt>', '</qt>', '<b>', '</b>', '<h1>', '</h1>', '<p>', '</p>,'\
	              '<em>', '</em>', '<ul>', '</ul>', '<i>', '</i>', '<td>', '</td>',\
		      '<tr>', '</tr>', '<u>', '</u>', '<li>', '</li>', '<br>', '</br>',\
		      '<filename>', '</filename>', '<para>', '</para>', '<application>',\
		      '</application>', '<resource>', '</resource>', '<warning>',\
		      '</warning>', '<command>', '</command>', '<interface>', '</interface>',\
		      '<item>', '<item>', '<list>', '</list>', '<icode>', '</icode>',\
		      '<email>', '</email>', '<message>', '</message>', '<note>', '</note']:
                s=s.replace(i,'')
#markupfilter ????
#<link linkend \"foo\">foo</link>????
	return s
        #leerzeichen an anfang/ende + mehrfache in mitte hier ausfiltern?
        # noch ausfiltern:
        # Kommentar '_: abcd\n' ok ist gefiltert
#    def buildmsgdict(self): 
#        self.msgfilterdict=getmsgfilterdict([self.filename])
    def is_desktop_file(self):
        if self.gettexttyp=='kde4':
            return False
        else:
            if "desktop_" in  self.name: #nur für kde3
                return True
            else:
                return False

def guidocpo(filename,backup=False):
        f=open(filename).read()
        if 'msgid' and 'msgstr' in f:
            if '#| msgid ' in f or 'msgctxt' in f or 'msgid_plural' in f: gettexttyp='kde4'
            else: gettexttyp='kde3'
            if '#. Tag:' in f:
                return docpo(filename,gettexttyp,backup)
            else:
                return guipo(filename,gettexttyp,backup)

class msgfilterdict:
    def __init__(self,pofilelist):
        self.msgdict={}
        for j in pofilelist:
            po=guidocpo(j)
            for i in po.messagerange():
                if not po.is_plural(i): #wg fehler bei pluralformen!!!
                    if po.is_translation(i) and po.is_translated(i) and not po.is_fuzzy(i):
                        if self.msgdict.has_key(po.msgidfilter[i]):
	                    if po.msgstrfilter[i] not in self.msgdict[po.msgidfilter[i]][0]:
		                m, m2 = self.msgdict[po.msgidfilter[i]][0], po.msgstrfilter[i]
		                m.append(m2)
		                na,na2 = self.msgdict[po.msgidfilter[i]][1], po.name
				na.append(na2)
		                nr,nr2 = self.msgdict[po.msgidfilter[i]][2], i
				nr.append(nr2)
				self.msgdict[po.msgidfilter[i]]=[m, na, nr]
                        else:
		            self.msgdict[po.msgidfilter[i]]=[[po.msgstrfilter[i]],[po.name],[i]]

class msgidlist:
    def __init__(self,pofilelist):
        self.msgidlist=[]
        for j in pofilelist:
            po=guidocpo(j)
            for i in po.messagerange():
#                if not po.is_plural(i): #wg fehler bei pluralformen!!!
                        if po.msgid[i] not in self.msgidlist:
		            self.msgidlist.append(po.msgid[i])

#def getmsgfilterdict(pofilelist): 
#    msgfilterdict={}
#    for i in pofilelist:
#        po=guidocpo(i)
#        for i in po.messagerange():#
#            if not po.is_plural(i): #wg fehler bei pluralformen!!!
#                if po.is_translation(i) and po.is_translated(i) and not po.is_fuzzy(i):
#                    msgfilterdict[po.msgidfilter[i]]=po.msgstrfilter[i]
#    return msgfilterdict
#        #bei Mehrfachkeys (kdelibs.po Move oder Print) wird nur der letze Wert eingetragen!!!!!!!

# Encoding der Standardausgabe und des Dateisystems herausfinden 
stdout_encoding = sys.stdout.encoding or sys.getfilesystemencoding() 
fs_encoding = sys.getfilesystemencoding() 
#print  stdout_encoding
#print fs_encoding

# Kompletten Kommandozeilenaufruf übernehmen und nach Unicode umwandeln 
#cmd_original = " ".join(sys.argv) 
#cmd_unicode = cmd_original.decode(fs_encoding) 
 
# String verändern und wieder an die Kommandozeile übergeben 
#cmd_unicode += u" (öäü)" 
#print cmd_unicode.encode(stdout_encoding)

if __name__ == "__main__":
    print "checkutils.py is no executable, just a python module with all common functions and objects,"+\
          " used by kzwiebelfisch.py and the other check*.py's"
